"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = cloneElement;

var _host = _interopRequireDefault(require("./vdom/host"));

var _isValidElement = _interopRequireDefault(require("./isValidElement"));

var _createElement = require("./createElement");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function cloneElement(element, config) {
  if (!(0, _isValidElement.default)(element)) {
    throw Error('cloneElement: not a valid element.' + (0, _createElement.getRenderErrorInfo)());
  } // Original props are copied


  var props = Object.assign({}, element.props); // Reserved names are extracted

  var key = element.key;
  var ref = element.ref; // Owner will be preserved, unless ref is overridden

  var owner = element._owner;

  if (config) {
    // Should reset ref and owner if has a new ref
    if (config.ref !== undefined) {
      ref = config.ref;
      owner = _host.default.component;
    }

    if (config.key !== undefined) {
      key = String(config.key);
    } // Resolve default props


    var defaultProps;

    if (element.type && element.type.defaultProps) {
      defaultProps = element.type.defaultProps;
    } // Remaining properties override existing props


    var propName;

    for (propName in config) {
      if (config.hasOwnProperty(propName) && !_createElement.RESERVED_PROPS.hasOwnProperty(propName)) {
        if (config[propName] === undefined && defaultProps !== undefined) {
          // Resolve default props
          props[propName] = defaultProps[propName];
        } else {
          props[propName] = config[propName];
        }
      }
    }
  }

  for (var _len = arguments.length, children = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments[_key];
  }

  if (children.length) {
    props.children = (0, _createElement.flattenChildren)(children);
  }

  return new _createElement.Element(element.type, key, ref, props, owner);
}

;