"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _reactive = _interopRequireDefault(require("./reactive"));

var _updater = _interopRequireDefault(require("./updater"));

var _host = _interopRequireDefault(require("./host"));

var _ref = _interopRequireDefault(require("./ref"));

var _instantiateComponent = _interopRequireDefault(require("./instantiateComponent"));

var _shouldUpdateComponent = _interopRequireDefault(require("./shouldUpdateComponent"));

var _shallowEqual = _interopRequireDefault(require("./shallowEqual"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function performInSandbox(fn, instance, callback) {
  try {
    return fn();
  } catch (e) {
    if (callback) {
      callback(e);
    } else {
      handleError(instance, e);
    }
  }
}

function handleError(instance, error) {
  var boundary;

  while (instance) {
    if (typeof instance.componentDidCatch === 'function') {
      boundary = instance;
      break;
    } else if (instance._internal && instance._internal._parentInstance) {
      instance = instance._internal._parentInstance;
    } else {
      break;
    }
  }

  if (boundary) {
    // should not attempt to recover an unmounting error boundary
    var boundaryInternal = boundary._internal;

    if (boundaryInternal) {
      var callbackQueue = boundaryInternal._pendingCallbacks || (boundaryInternal._pendingCallbacks = []);
      callbackQueue.push(function () {
        return boundary.componentDidCatch(error);
      });
    }
  } else {
    if (_host.default.sandbox) {
      setTimeout(function () {
        throw error;
      }, 0);
    } else {
      throw error;
    }
  }
}

var measureLifeCycle;

if (process.env.NODE_ENV !== 'production') {
  measureLifeCycle = function measureLifeCycle(callback, instanceID, type) {
    _host.default.measurer && _host.default.measurer.beforeLifeCycle(instanceID, type);
    callback();
    _host.default.measurer && _host.default.measurer.afterLifeCycle(instanceID, type);
  };
}
/**
 * Composite Component
 */


var CompositeComponent =
/*#__PURE__*/
function () {
  function CompositeComponent(element) {
    _classCallCheck(this, CompositeComponent);

    this._currentElement = element;
  }

  _createClass(CompositeComponent, [{
    key: "getName",
    value: function getName() {
      var type = this._currentElement.type;
      var constructor = this._instance && this._instance.constructor;
      return type.displayName || constructor && constructor.displayName || type.name || constructor && constructor.name || null;
    }
  }, {
    key: "mountComponent",
    value: function mountComponent(parent, parentInstance, context, childMounter) {
      var _this = this;

      this._parent = parent;
      this._parentInstance = parentInstance;
      this._context = context;
      this._mountID = _host.default.mountID++;
      this._updateCount = 0;

      if (process.env.NODE_ENV !== 'production') {
        _host.default.measurer && _host.default.measurer.beforeMountComponent(this._mountID, this);
      }

      var currentElement = this._currentElement;
      var Component = currentElement.type;
      var ref = currentElement.ref;
      var publicProps = currentElement.props;
      var isClass = Component.prototype;
      var isComponentClass = isClass && Component.prototype.isComponentClass; // Class stateless component without state but have lifecycles

      var isStatelessClass = isClass && Component.prototype.render; // Context process

      var publicContext = this._processContext(context); // Initialize the public class


      var instance;
      var renderedElement;

      try {
        if (isComponentClass || isStatelessClass) {
          // Component instance
          instance = new Component(publicProps, publicContext, _updater.default);
        } else if (typeof Component === 'function') {
          // Functional reactive component with hooks
          instance = new _reactive.default(Component, ref);
        } else {
          throw new Error("Invalid component type: ".concat(Component, ". (current: ").concat(_typeof(Component) === 'object' && Object.keys(Component) || _typeof(Component), ")"));
        }
      } catch (e) {
        handleError(parentInstance, e);
        return instance;
      } // These should be set up in the constructor, but as a convenience for
      // simpler class abstractions, we set them up after the fact.


      instance.props = publicProps;
      instance.context = publicContext;
      instance.refs = {}; // Inject the updater into instance

      instance.updater = _updater.default;
      instance._internal = this;
      this._instance = instance; // Init state, must be set to an object or null

      var initialState = instance.state;

      if (initialState === undefined) {
        // TODO clone the state?
        instance.state = initialState = null;
      }

      var error = null;

      var errorCallback = function errorCallback(e) {
        error = e;
      };

      if (instance.componentWillMount) {
        performInSandbox(function () {
          if (process.env.NODE_ENV !== 'production') {
            measureLifeCycle(function () {
              instance.componentWillMount();
            }, _this._mountID, 'componentWillMount');
          } else {
            instance.componentWillMount();
          }
        }, instance, errorCallback);
      }

      if (renderedElement == null) {
        _host.default.component = this; // Process pending state when call setState in componentWillMount

        instance.state = this._processPendingState(publicProps, publicContext);
        performInSandbox(function () {
          if (process.env.NODE_ENV !== 'production') {
            measureLifeCycle(function () {
              renderedElement = instance.render();
            }, _this._mountID, 'render');
          } else {
            renderedElement = instance.render();
          }
        }, instance, errorCallback);
        _host.default.component = null;
      }

      this._renderedComponent = (0, _instantiateComponent.default)(renderedElement);

      this._renderedComponent.mountComponent(this._parent, instance, this._processChildContext(context), childMounter);

      if (error) {
        handleError(instance, error);
      }

      if (!this._currentElement.type.forwardRef && ref) {
        _ref.default.attach(currentElement._owner, ref, this);
      }

      if (instance.componentDidMount) {
        performInSandbox(function () {
          if (process.env.NODE_ENV !== 'production') {
            measureLifeCycle(function () {
              instance.componentDidMount();
            }, _this._mountID, 'componentDidMount');
          } else {
            instance.componentDidMount();
          }
        }, instance);
      } // Trigger setState callback in componentWillMount or boundary callback after rendered


      var callbacks = this._pendingCallbacks;

      if (callbacks) {
        this._pendingCallbacks = null;

        _updater.default.runCallbacks(callbacks, instance);
      }

      _host.default.hook.Reconciler.mountComponent(this);

      if (process.env.NODE_ENV !== 'production') {
        _host.default.measurer && _host.default.measurer.afterMountComponent(this._mountID);
      }

      return instance;
    }
  }, {
    key: "unmountComponent",
    value: function unmountComponent(notRemoveChild) {
      var instance = this._instance;

      if (!instance) {
        return;
      }

      if (instance.componentWillUnmount) {
        performInSandbox(function () {
          instance.componentWillUnmount();
        }, instance);
      }

      _host.default.hook.Reconciler.unmountComponent(this);

      instance._internal = null;

      if (this._renderedComponent != null) {
        var ref = this._currentElement.ref;

        if (!this._currentElement.type.forwardRef && ref) {
          _ref.default.detach(this._currentElement._owner, ref, this);
        }

        this._renderedComponent.unmountComponent(notRemoveChild);

        this._renderedComponent = null;
        this._instance = null;
      }

      this._currentElement = null;
      this._parentInstance = null; // Reset pending fields
      // Even if this component is scheduled for another update in ReactUpdates,
      // it would still be ignored because these fields are reset.

      this._pendingStateQueue = null;
      this._pendingForceUpdate = false; // These fields do not really need to be reset since this object is no
      // longer accessible.

      this._context = null;
    }
    /**
     * Filters the context object to only contain keys specified in
     * `contextTypes`
     */

  }, {
    key: "_processContext",
    value: function _processContext(context) {
      var Component = this._currentElement.type;
      var contextTypes = Component.contextTypes;

      if (!contextTypes) {
        return {};
      }

      var maskedContext = {};

      for (var contextName in contextTypes) {
        maskedContext[contextName] = context[contextName];
      }

      return maskedContext;
    }
  }, {
    key: "_processChildContext",
    value: function _processChildContext(currentContext) {
      var instance = this._instance;
      var childContext = instance.getChildContext && instance.getChildContext();

      if (childContext) {
        return Object.assign({}, currentContext, childContext);
      }

      return currentContext;
    }
  }, {
    key: "_processPendingState",
    value: function _processPendingState(props, context) {
      var instance = this._instance;
      var queue = this._pendingStateQueue;

      if (!queue) {
        return instance.state;
      } // Reset pending queue


      this._pendingStateQueue = null;
      var nextState = Object.assign({}, instance.state);

      for (var i = 0; i < queue.length; i++) {
        var partial = queue[i];
        Object.assign(nextState, typeof partial === 'function' ? partial.call(instance, nextState, props, context) : partial);
      }

      return nextState;
    }
  }, {
    key: "updateComponent",
    value: function updateComponent(prevElement, nextElement, prevUnmaskedContext, nextUnmaskedContext) {
      var instance = this._instance;

      if (process.env.NODE_ENV !== 'production') {
        _host.default.measurer && _host.default.measurer.beforeUpdateComponent(this._mountID, this);
      }

      if (!instance) {
        console.error("Update component '".concat(this.getName(), "' that has already been unmounted (or failed to mount)."));
      }

      var willReceive = false;
      var nextContext;
      var nextProps; // Determine if the context has changed or not

      if (this._context === nextUnmaskedContext) {
        nextContext = instance.context;
      } else {
        nextContext = this._processContext(nextUnmaskedContext);
        willReceive = true;
      } // Distinguish between a props update versus a simple state update


      if (prevElement === nextElement) {
        // Skip checking prop types again -- we don't read component.props to avoid
        // warning for DOM component props in this upgrade
        nextProps = nextElement.props;
      } else {
        nextProps = nextElement.props;
        willReceive = true;
      }

      var hasReceived = willReceive && instance.componentWillReceiveProps;

      if (hasReceived) {
        // Calling this.setState() within componentWillReceiveProps will not trigger an additional render.
        this._pendingState = true;
        performInSandbox(function () {
          instance.componentWillReceiveProps(nextProps, nextContext);
        }, instance);
        this._pendingState = false;
      } // Update refs


      if (this._currentElement.type.forwardRef) {
        instance.prevForwardRef = prevElement.ref;
        instance.forwardRef = nextElement.ref;
      } else {
        _ref.default.update(prevElement, nextElement, this);
      } // Shoud update always default


      var shouldUpdate = true;
      var prevProps = instance.props;
      var prevState = instance.state; // TODO: could delay execution processPendingState

      var nextState = this._processPendingState(nextProps, nextContext); // ShouldComponentUpdate is not called when forceUpdate is used


      if (!this._pendingForceUpdate) {
        if (instance.shouldComponentUpdate) {
          shouldUpdate = performInSandbox(function () {
            return instance.shouldComponentUpdate(nextProps, nextState, nextContext);
          }, instance);
        } else if (instance.isPureComponentClass) {
          shouldUpdate = !(0, _shallowEqual.default)(prevProps, nextProps) || !(0, _shallowEqual.default)(prevState, nextState);
        }
      }

      if (shouldUpdate) {
        this._pendingForceUpdate = false; // Will set `this.props`, `this.state` and `this.context`.

        var prevContext = instance.context; // Cannot use this.setState() in componentWillUpdate.
        // If need to update state in response to a prop change, use componentWillReceiveProps instead.

        performInSandbox(function () {
          if (instance.componentWillUpdate) {
            instance.componentWillUpdate(nextProps, nextState, nextContext);
          }
        }, instance); // Replace with next

        this._currentElement = nextElement;
        this._context = nextUnmaskedContext;
        instance.props = nextProps;
        instance.state = nextState;
        instance.context = nextContext;

        this._updateRenderedComponent(nextUnmaskedContext);

        performInSandbox(function () {
          if (instance.componentDidUpdate) {
            instance.componentDidUpdate(prevProps, prevState, prevContext);
          }
        }, instance);
        this._updateCount++;
      } else {
        // If it's determined that a component should not update, we still want
        // to set props and state but we shortcut the rest of the update.
        this._currentElement = nextElement;
        this._context = nextUnmaskedContext;
        instance.props = nextProps;
        instance.state = nextState;
        instance.context = nextContext;
      } // Flush setState callbacks set in componentWillReceiveProps or boundary callback


      var callbacks = this._pendingCallbacks;

      if (callbacks) {
        this._pendingCallbacks = null;

        _updater.default.runCallbacks(callbacks, instance);
      }

      if (process.env.NODE_ENV !== 'production') {
        _host.default.measurer && _host.default.measurer.afterUpdateComponent(this._mountID);
      }

      _host.default.hook.Reconciler.receiveComponent(this);
    }
    /**
     * Call the component's `render` method and update the DOM accordingly.
     */

  }, {
    key: "_updateRenderedComponent",
    value: function _updateRenderedComponent(context) {
      var _this2 = this;

      var prevRenderedComponent = this._renderedComponent;
      var prevRenderedElement = prevRenderedComponent._currentElement;
      var instance = this._instance;
      var nextRenderedElement;
      _host.default.component = this;
      performInSandbox(function () {
        if (process.env.NODE_ENV !== 'production') {
          measureLifeCycle(function () {
            nextRenderedElement = instance.render();
          }, _this2._mountID, 'render');
        } else {
          nextRenderedElement = instance.render();
        }
      }, instance);
      _host.default.component = null;

      if ((0, _shouldUpdateComponent.default)(prevRenderedElement, nextRenderedElement)) {
        var prevRenderedUnmaskedContext = prevRenderedComponent._context;

        var nextRenderedUnmaskedContext = this._processChildContext(context);

        if (prevRenderedElement !== nextRenderedElement || prevRenderedUnmaskedContext !== nextRenderedUnmaskedContext) {
          prevRenderedComponent.updateComponent(prevRenderedElement, nextRenderedElement, prevRenderedUnmaskedContext, nextRenderedUnmaskedContext);
        }

        if (process.env.NODE_ENV !== 'production') {
          _host.default.measurer && _host.default.measurer.recordOperation({
            instanceID: this._mountID,
            type: 'update component',
            payload: {}
          });
        }
      } else {
        var oldChild = prevRenderedComponent.getNativeNode();
        prevRenderedComponent.unmountComponent(true);
        this._renderedComponent = (0, _instantiateComponent.default)(nextRenderedElement);

        this._renderedComponent.mountComponent(this._parent, instance, this._processChildContext(context), function (newChild, parent) {
          // TODO: Duplicate code in native component file
          if (!Array.isArray(newChild)) {
            newChild = [newChild];
          } // oldChild or newChild all maybe fragment


          if (!Array.isArray(oldChild)) {
            oldChild = [oldChild];
          } // If newChild count large then oldChild


          var lastNewChild;

          for (var i = 0; i < newChild.length; i++) {
            var child = newChild[i];

            if (oldChild[i]) {
              _host.default.driver.replaceChild(child, oldChild[i]);
            } else if (lastNewChild) {
              _host.default.driver.insertAfter(child, lastNewChild);
            } else {
              _host.default.driver.appendChild(child, parent);
            }

            lastNewChild = child;
          } // If newChild count less then oldChild


          if (newChild.length < oldChild.length) {
            for (var _i = newChild.length; _i < oldChild.length; _i++) {
              _host.default.driver.removeChild(oldChild[_i]);
            }
          }
        });
      }
    }
  }, {
    key: "getNativeNode",
    value: function getNativeNode() {
      var renderedComponent = this._renderedComponent;

      if (renderedComponent) {
        return renderedComponent.getNativeNode();
      }
    }
  }, {
    key: "getPublicInstance",
    value: function getPublicInstance() {
      var instance = this._instance; // The Stateless components cannot be given refs

      if (instance instanceof _reactive.default) {
        return null;
      }

      return instance;
    }
  }]);

  return CompositeComponent;
}();

var _default = CompositeComponent;
exports.default = _default;