"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _createElement = _interopRequireDefault(require("../createElement"));

var _unmountComponentAtNode = _interopRequireDefault(require("../unmountComponentAtNode"));

var _instantiateComponent = _interopRequireDefault(require("./instantiateComponent"));

var _shouldUpdateComponent = _interopRequireDefault(require("./shouldUpdateComponent"));

var _root = _interopRequireDefault(require("./root"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Instance manager
 */
var KEY = '$$instance';
var _default = {
  set: function set(node, instance) {
    if (!node[KEY]) {
      node[KEY] = instance; // Record root instance to roots map

      if (instance.rootID) {
        _host.default.rootInstances[instance.rootID] = instance;
        _host.default.rootComponents[instance.rootID] = instance._internal;
      }
    }
  },
  get: function get(node) {
    return node[KEY];
  },
  remove: function remove(node) {
    var instance = this.get(node);

    if (instance) {
      node[KEY] = null;

      if (instance.rootID) {
        delete _host.default.rootComponents[instance.rootID];
        delete _host.default.rootInstances[instance.rootID];
      }
    }
  },
  mount: function mount(element, container, parentInstance) {
    if (process.env.NODE_ENV !== 'production') {
      _host.default.measurer && _host.default.measurer.beforeRender();
    } // Before render callback


    _host.default.driver.beforeRender && _host.default.driver.beforeRender(); // Real native root node is body

    if (container == null) {
      container = _host.default.driver.createBody();
    } // Get the context from the conceptual parent component.


    var parentContext;

    if (parentInstance) {
      var parentInternal = parentInstance._internal;
      parentContext = parentInternal._processChildContext(parentInternal._context);
    }

    var prevRootInstance = this.get(container);
    var hasPrevRootInstance = prevRootInstance && prevRootInstance.isRootComponent;

    if (hasPrevRootInstance) {
      var prevRenderedComponent = prevRootInstance.getRenderedComponent();
      var prevElement = prevRenderedComponent._currentElement;

      if ((0, _shouldUpdateComponent.default)(prevElement, element)) {
        var prevUnmaskedContext = prevRenderedComponent._context;
        var nextUnmaskedContext = parentContext || prevUnmaskedContext;

        if (prevElement !== element || prevUnmaskedContext !== nextUnmaskedContext) {
          prevRenderedComponent.updateComponent(prevElement, element, prevUnmaskedContext, nextUnmaskedContext);
        }

        return prevRootInstance;
      } else {
        _host.default.hook.Reconciler.unmountComponent(prevRootInstance);

        (0, _unmountComponentAtNode.default)(container);
      }
    }

    var wrappedElement = (0, _createElement.default)(_root.default, null, element);
    var renderedComponent = (0, _instantiateComponent.default)(wrappedElement);
    var defaultContext = parentContext || {};
    var rootInstance = renderedComponent.mountComponent(container, null, defaultContext);
    this.set(container, rootInstance); // After render callback

    _host.default.driver.afterRender && _host.default.driver.afterRender(rootInstance); // Devtool render new root hook

    _host.default.hook.Mount._renderNewRootComponent(rootInstance._internal);

    if (process.env.NODE_ENV !== 'production') {
      _host.default.measurer && _host.default.measurer.afterRender();
    }

    return rootInstance;
  }
};
exports.default = _default;