"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _component = _interopRequireDefault(require("../component"));

var _scheduler = require("./scheduler");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var RE_RENDER_LIMIT = 24;
/**
 * Functional Reactive Component Class Wrapper
 */

var ReactiveComponent =
/*#__PURE__*/
function (_Component) {
  _inherits(ReactiveComponent, _Component);

  function ReactiveComponent(pureRender, ref) {
    var _this;

    _classCallCheck(this, ReactiveComponent);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ReactiveComponent).call(this)); // A pure function

    _this.pureRender = pureRender;
    _this.hooksIndex = 0;
    _this.hooks = {};
    _this.didMountHandlers = [];
    _this.didUpdateHandlers = [];
    _this.willUnmountHandlers = [];
    _this.isRenderScheduled = false;
    _this.numberOfReRenders = 0;

    if (pureRender.forwardRef) {
      _this.prevForwardRef = _this.forwardRef = ref;
    }

    var compares = pureRender.compares;

    if (compares) {
      _this.shouldComponentUpdate = function (nextProps) {
        // Process composed compare
        var arePropsEqual = true; // Compare push in and pop out

        for (var i = compares.length - 1; i > -1; i--) {
          if (arePropsEqual = compares[i](_this.props, nextProps)) {
            break;
          }

          ;
        }

        return !arePropsEqual || _this.prevForwardRef !== _this.forwardRef;
      };
    }

    return _this;
  }

  _createClass(ReactiveComponent, [{
    key: "getCurrentHookId",
    value: function getCurrentHookId() {
      return ++this.hooksIndex;
    }
  }, {
    key: "readContext",
    value: function readContext(context) {
      var _this2 = this;

      var Provider = context.Provider;
      var unmaskContext = this._internal._context;
      var contextProp = Provider.contextProp;
      var contextEmitter = unmaskContext[contextProp];

      if (contextEmitter) {
        var mountId = this._internal._mountID;

        if (!contextEmitter[mountId]) {
          // One context one updater bind
          contextEmitter[mountId] = {};

          var contextUpdater = function contextUpdater(newContext) {
            if (newContext !== contextEmitter[mountId].renderedContext) {
              _this2.update();
            }
          };

          contextEmitter.on(contextUpdater);
          this.willUnmountHandlers.push(function () {
            delete contextEmitter[mountId];
            contextEmitter.off(contextUpdater);
          });
        }

        return contextEmitter[mountId].renderedContext = contextEmitter.value;
      }

      return Provider.defaultValue;
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.didMountHandlers.forEach(function (handler) {
        return handler();
      });
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      this.didUpdateHandlers.forEach(function (handler) {
        return handler();
      });
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.willUnmountHandlers.forEach(function (handler) {
        return handler();
      });
    } // Async update

  }, {
    key: "update",
    value: function update() {
      var _this3 = this;

      (0, _scheduler.scheduleImmediateCallback)(function () {
        return _this3.forceUpdate();
      });
    }
  }, {
    key: "render",
    value: function render() {
      if (process.env.NODE_ENV !== 'production') {
        _host.default.measurer && _host.default.measurer.beforeRender();
      }

      this.hooksIndex = 0;
      this.numberOfReRenders = 0;
      this.isRenderScheduled = false;
      var children = this.pureRender(this.props, this.forwardRef ? this.forwardRef : this.context);

      while (this.isRenderScheduled) {
        this.numberOfReRenders++;

        if (this.numberOfReRenders > RE_RENDER_LIMIT) {
          throw new Error('Too many re-renders, the number of renders is limited to prevent an infinite loop.');
        }

        this.hooksIndex = 0;
        this.isRenderScheduled = false;
        children = this.pureRender(this.props, this.forwardRef ? this.forwardRef : this.context);
      }

      return children;
    }
  }]);

  return ReactiveComponent;
}(_component.default);

var _default = ReactiveComponent;
exports.default = _default;